#include <windows.h>
#include <math.h>

#define				PI					(3.1415926535897932384626433832795)
#define				PI2					(PI * 2.0)

#define				W					128
#define				H					128
#define				SW					(W*1)
#define				SH					(H*1)

#define				R					(50)

typedef	unsigned long		u32;
typedef	float				f32;

static char			g_szWindowName[]	= "dib";
static char			g_szWindowClass[]	= "dib32";
static HINSTANCE	g_hInst;
static HWND			g_hWnd;

static BITMAPINFO	g_bi;
static u32 *		p_dib;

static f32 dist( f32 x0, f32 y0, f32 x1, f32 y1 )
{
	f32		dx, dy;
	dx = x1 - x0;
	dy = y1 - y0;
	return sqrt( dx * dx + dy * dy );
}

static void test( void )
{
	u32			c;
	f32			d, a;
	int			x, y;
	f32			cx, cy;
	static f32	phase	= 0.0f;

	cx = cos( phase ) * 10.0f + (W / 2);
	cy = sin( phase ) * 10.0f + (H / 2);

	for ( y = 0; y < H; y++ )
	for ( x = 0; x < W; x++ ) {
		d = dist( cx, cy, x, y );
		if ( d < R ) {
			if ( d > (R - 1) ) {
				a = 1.0f - (d - (R - 1));
			} else {
				a = 1.0f;
			}
			c = 255 - (u32)floor( a * 255.0f );
			c = (c << 16) | (c << 8) | c;
			p_dib[W * y + x] = c;
		}
	}

	phase += 0.01f;
}

static LRESULT CALLBACK wnd_proc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch ( uMsg ) {
		case WM_NCHITTEST: return HTCAPTION;

		case WM_PAINT: {
			PAINTSTRUCT	ps;
			BeginPaint( hWnd, &ps );
			StretchDIBits( ps.hdc, 0, 0, SW, SH, 0, 0, W, H, p_dib, &g_bi, DIB_RGB_COLORS, SRCCOPY );
			EndPaint( hWnd, &ps );
		} return 0;

		case WM_KEYDOWN: if ( (int)wParam != VK_ESCAPE ) break;
		case WM_DESTROY:
			PostQuitMessage( 0 );
			return 0;
	}
	return DefWindowProc( hWnd, uMsg, wParam, lParam );
}

int APIENTRY WinMain( HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
	MSG			msg;
	WNDCLASS	wc;
	HDC			hdc;

	g_hInst = hInst;

	memset( &wc, 0, sizeof( WNDCLASS ) );
	wc.style			= CS_OWNDC | CS_DBLCLKS;
	wc.lpfnWndProc		= wnd_proc;
	wc.hInstance		= hInst;
	wc.lpszClassName	= g_szWindowClass;
	wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
	RegisterClass( &wc );

	g_hWnd = CreateWindowEx( WS_EX_TOPMOST,
		g_szWindowClass, g_szWindowName, WS_POPUP | WS_VISIBLE,
		(GetSystemMetrics( SM_CXSCREEN ) >> 1) - (SW >> 1),
		(GetSystemMetrics( SM_CYSCREEN ) >> 1) - (SH >> 1),
		SW, SH, NULL, NULL, hInst, NULL );

	memset( &g_bi, 0, sizeof( BITMAPINFO ) );
	g_bi.bmiHeader.biSize			= sizeof( BITMAPINFOHEADER );
	g_bi.bmiHeader.biWidth			= W;
	g_bi.bmiHeader.biHeight			= -H;
	g_bi.bmiHeader.biPlanes			= 1;
	g_bi.bmiHeader.biBitCount		= 32;
	g_bi.bmiHeader.biCompression	= BI_RGB;

	p_dib = malloc( W * H * 4 );

	memset( &msg, 0, sizeof( MSG ) );
	while ( msg.message != WM_QUIT ) {
		if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) {
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		} else {
			memset( p_dib, 255, W * H * 4 );
			test();
			hdc = GetWindowDC( g_hWnd );
			StretchDIBits( hdc, 0, 0, SW, SH, 0, 0, W, H, p_dib, &g_bi, DIB_RGB_COLORS, SRCCOPY );
			ReleaseDC( g_hWnd, hdc );
			Sleep( 50 );
		}
	}

	free( p_dib );
	DestroyWindow( g_hWnd );
	UnregisterClass( g_szWindowClass, g_hInst );
	return 0;
}
